//
// A sample which uses the fractal library
//
//                              by Sebastian Mihai, 2013
//


// onload method
function smOnload() {

	// point the fractal generator at the proper DOM element
	var fractal = new Fractal( "smCanvas", "Koch Snowflake, by Sebastian Mihai" );
	
	// we'll generate up to this depth
	fractal.SetMaxDepth( 4 );

	var cx = fractal.Width() / 2;
	var cy = fractal.Height() / 2;
	
	var initialSegmentLength = 500;
	var yShift = 140;
	
	// define initial triangle via its three vertices
	var v1 = new KochVertex( cx - initialSegmentLength/2, cy + yShift );
	var v2 = new KochVertex( cx + initialSegmentLength/2, cy + yShift );
	
	var height = Math.sqrt( 
		Math.pow( initialSegmentLength, 2 ) -
		Math.pow( initialSegmentLength/2, 2 )
		);
	var v3 = new KochVertex( cx, cy + yShift - height );
	
	var initialSegment1 = new KochQueueItem( v1, v2, 0 );
	var initialSegment2 = new KochQueueItem( v2, v3, 0 );
	var initialSegment3 = new KochQueueItem( v3, v1, 0 );
	
	// draw the initial three segments
	fractal.DrawLine( v1.X(), v1.Y(), v2.X(), v2.Y() );
	fractal.DrawLine( v2.X(), v2.Y(), v3.X(), v3.Y() );
	fractal.DrawLine( v3.X(), v3.Y(), v1.X(), v1.Y() );
	
	// and push the three segments
	fractal.Queue().Push( initialSegment1 );
	fractal.Queue().Push( initialSegment2 );
	fractal.Queue().Push( initialSegment3 );

	// this function will be called by the fractal generator for each node
	var delegate = function( theFractal, theQueue, node, depth ) {

		var v1 = node.V1();
		var v2 = node.V2();

		// Transform
		// 
		// v1----------v2
		//
		//      to
		//
		//       q
		//      / \
		// v1--p---r--v2
		//
		
		var pqr = splitSegment( v1, v2 );
		var p = pqr[0];
		var q = pqr[1];
		var r = pqr[2];
		
		theFractal.DrawLine( p.X(), p.Y(), q.X(), q.Y() );
		theFractal.DrawLine( q.X(), q.Y(), r.X(), r.Y() );
		
		// "erase" bottom segment...
		for( var c = 0; c < 10; c++ ) {
			theFractal.DrawLine( p.X(), p.Y(), r.X(), r.Y(), "#000000" );
			}
			
		// see diagram above to see which segments constitute our next iteration
		var v1_p = new KochQueueItem( v1, p, depth+1 );
		var r_v2 = new KochQueueItem( r, v2, depth+1 );
		var p_q = new KochQueueItem( p, q, depth+1 );
		var q_r = new KochQueueItem( q, r, depth+1 );
		
		theQueue.Push( v1_p );
		theQueue.Push( p_q );
		theQueue.Push( q_r );
		theQueue.Push( r_v2 );
		
	}

	// tell fractal generator about our delegate
	fractal.SetDelegate( delegate );
	
	// render fractal in an progressive, animated fashion
	// ( for a direct rendering, use fractal.Render() )
	fractal.RenderProgressively( 50 );
	//fractal.Render();
}

// split a segment
function splitSegment( v1, v2 ) {
	
	var u_vector = new KochVertex( 
		v2.X() - v1.X(),
		v2.Y() - v1.Y()
		);
		
	var v_vector = new KochVertex( 
		v1.Y() - v2.Y(),
		v2.X() - v1.X()
		);
		
	var p = new KochVertex(
		v1.X() + (1/3) * u_vector.X(),
		v1.Y() + (1/3) * u_vector.Y()
		);
	
	var q = new KochVertex(
		v1.X() + (1/2) * u_vector.X() + ( Math.sqrt(3) / 6 ) * v_vector.X(),
		v1.Y() + (1/2) * u_vector.Y() + ( Math.sqrt(3) / 6 ) * v_vector.Y()
		);
		
	var r = new KochVertex(
		v1.X() + (2/3) * u_vector.X(),
		v1.Y() + (2/3) * u_vector.Y()
		);
		
	return new Array( p, q, r );
}


// a "class" representing a vertex
function KochVertex( x, y ) {

	this.m_x = x;
	this.m_y = y;
	
	KochVertex.prototype.X = function() {
		return this.m_x;
	}
	
	KochVertex.prototype.Y = function() {
		return this.m_y;
	}
}


// a "class" for a Koch Snowflake segment node in a recursion iteration
// it stores a line segment, defined by two vertices
function KochQueueItem( v1, v2, depth ) {

	this.m_v1 = v1;
	this.m_v2 = v2;
	this.m_depth = depth;
	
	KochQueueItem.prototype.V1 = function() {
		return this.m_v1;
	}
	
	KochQueueItem.prototype.V2 = function() {
		return this.m_v2;
	}

	KochQueueItem.prototype.Depth = function() {
		return this.m_depth;
	}
}
